@@ -11,7 +11,7 @@ use URPM::Resolve;
 use URPM::Signature;
 
 our @ISA = qw(DynaLoader);
-our $VERSION = '4.10';
+our $VERSION = '4.16';
 
 URPM->bootstrap($VERSION);
 
@@ -292,7 +292,7 @@ URPM - Manipulate RPM files and headers
 =head1 DESCRIPTION
 
 The URPM module allows you to manipulate RPM files, RPM header files and
-hdlist files and manage them in memory. It is notably used by the C<urpmi>
+hdlist files and manage them in memory. It is notably used by the L<urpmi>
 utility. It provides four classes : C<URPM>, C<URPM::DB>, C<URPM::Package>,
 and C<URPM::Transaction>.
 
@@ -471,10 +471,10 @@ Quite similar to C<traverse_tag>, but stops when $callback returns true.
 
 (also note that only one $name is handled)
 
-=item $db->create_transaction($prefix)
+=item $db->create_transaction()
 
 Creates and returns a new transaction (an C<URPM::Transaction> object) on the
-specified DB. For $prefix, cf L<open>.
+specified DB.
 
 =back
 
@@ -532,6 +532,8 @@ This is only used when faking a URPM::DB: $urpm can be used as-a $db
 
 =item $package->epoch()
 
+=item $package->EVR()
+
 =item $package->excludearchs()
 
 =item $package->exclusivearchs()
@@ -764,7 +766,6 @@ Recognized options are:
     nosize => 0 / 1
     noscripts => 0 / 1
     oldpackage => 0 / 1
-    repackage => 0 / 1
     test => 0 / 1
     translate_message => 1
 
@@ -887,7 +888,9 @@ Copyright 2002, 2003, 2004, 2005 MandrakeSoft SA
 
 Copyright 2005, 2006, 2007, 2008 Mandriva SA
 
-FranE<ccedil>ois Pons (original author), Rafael Garcia-Suarez, Pixel <pixel@mandriva.com> (current maintainer)
+Copyright 2011, 2012, Mangeia
+
+FranE<ccedil>ois Pons (original author), Rafael Garcia-Suarez, Pixel, Thierry Vignaud <tv@mageia.org> (current maintainer)
 
 This library is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
