@@ -11,6 +11,11 @@ use Config;
 # perl_checker: require URPM
 
 #- a few functions from MDK::Common copied here:
+sub any(&@) {
+    my $f = shift;
+    $f->($_) and return 1 foreach @_;
+    0;
+}
 sub listlength {
     my (@l) = @_;
     scalar @l;
@@ -63,7 +68,7 @@ sub removed_or_obsoleted_packages {
 #- (nb: see also find_required_package())
 #-
 #- side-effects: none
-sub find_candidate_packages_ {
+sub find_candidate_packages {
     my ($urpm, $id_prop, $o_rejected) = @_;
     my @packages;
 
@@ -90,19 +95,6 @@ sub find_candidate_packages_ {
     @packages;
 }
 
-#- deprecated, use find_candidate_packages_() directly
-#-
-#- side-effects: none
-sub find_candidate_packages {
-    my ($urpm, $id_prop, $o_rejected) = @_;
-
-    my %packages;
-    foreach (find_candidate_packages_($urpm, $id_prop, $o_rejected)) {
-	push @{$packages{$_->name}}, $_;
-    }
-    \%packages;
-}
-
 #- returns the "arch" of package $n in rpm db
 sub get_installed_arch {
     my ($db, $n) = @_;
@@ -125,11 +117,12 @@ my %installed_arch;
 #- side-effects: none (but uses a cache)
 sub strict_arch_check_installed {
     my ($db, $pkg) = @_;
-    if ($pkg->arch ne 'src' && $pkg->arch ne 'noarch') {
+    my $arch = $pkg->arch;
+    if ($arch ne 'src' && $arch ne 'noarch') {
 	my $n = $pkg->name;
 	defined $installed_arch{$n} or $installed_arch{$n} = get_installed_arch($db, $n);
 	if ($installed_arch{$n} && $installed_arch{$n} ne 'noarch') {
-	    $pkg->arch eq $installed_arch{$n} or return;
+	    $arch eq $installed_arch{$n} or return;
 	}
     }
     1;
@@ -141,9 +134,11 @@ sub strict_arch_check_installed {
 #- side-effects: none
 sub strict_arch_check {
     my ($installed_pkg, $pkg) = @_;
-    if ($pkg->arch ne 'src' && $pkg->arch ne 'noarch') {
-	if ($installed_pkg->arch ne 'noarch') {
-	    $pkg->arch eq $installed_pkg->arch or return;
+    my $arch = $pkg->arch;
+    if ($arch ne 'src' && $arch ne 'noarch') {
+	my $inst_arch = $installed_pkg->arch;
+	if ($inst_arch ne 'noarch') {
+	    $arch eq $inst_arch or return;
 	}
     }
     1;
@@ -189,12 +184,9 @@ sub provided_version_that_overlaps {
     $version;
 }
 
-#- deprecated function, use find_required_package()
-sub find_chosen_packages { &find_required_package }
-
 #- find the package (or packages) to install matching $id_prop
 #- returns (list ref of matches, list ref of preferred matches)
-#- (see also find_candidate_packages_())
+#- (see also find_candidate_packages())
 #-
 #- side-effects: flag_install, flag_upgrade (and strict_arch_check_installed cache)
 sub find_required_package {
@@ -359,7 +351,7 @@ sub _find_required_package__kmod {
     $choices->[0]->name =~ /^dkms-|-kernel-\d\./ or return;
 
     grep {
-	if (my ($_name, $version, $flavor, $release) = $_->name =~ /(.*)-kernel-(\d\..*)-(.*)-(.*)/) {
+	if (my ($version, $flavor, $release) = $_->name =~ /(:?.*)-kernel-(\d\..*)-(.*)-(.*)/) {
 	    my $kernel = "kernel-$flavor-$version-$release";
 	    _is_selected_or_installed($urpm, $db, $kernel);
 	} elsif ($_->name =~ /^dkms-/) {
@@ -388,7 +380,7 @@ sub _score_for_locales {
 	  } else {
 	      0; # bad locale
 	  }
-    } elsif (grep { /locales-en/ } @r) {
+    } elsif (any { /locales-en/ } @r) {
 	2; # 
     } else {
 	1;
@@ -499,14 +491,14 @@ sub unsatisfied_requires {
 	    #- check on installed system if a package which is not obsoleted is satisfying the require.
 	    my $satisfied = 0;
 	    if ($n =~ m!^/!) {
-		$db->traverse_tag('path', [ $n ], sub {
+		$db->traverse_tag_find('path', $n, sub {
 		    my ($p) = @_;
 		    exists $state->{rejected}{$p->fullname} and return;
 		    $state->{cached_installed}{$n}{$p->fullname} = undef;
 		    ++$satisfied;
 		});
 	    } else {
-		$db->traverse_tag('whatprovides', [ $n ], sub {
+		$db->traverse_tag_find('whatprovides', $n, sub {
 		    my ($p) = @_;
 		    exists $state->{rejected}{$p->fullname} and return;
 		    foreach ($p->provides) {
@@ -581,7 +573,7 @@ sub backtrack_selected {
 
 	    #- search for all possible packages, first is to try the selection, then if it is
 	    #- impossible, backtrack the origin.
-	    my @packages = find_candidate_packages_($urpm, $dep->{required});
+	    my @packages = find_candidate_packages($urpm, $dep->{required});
 
 	    foreach (@packages) {
 		    #- avoid dead loop.
@@ -882,13 +874,20 @@ sub resolve_requested {
     my @selected = resolve_requested__no_suggests($urpm, $db, $state, $requested, %options);
 
     if (!$options{no_suggests}) {
-	my @todo = @selected;
+        @selected = resolve_requested_suggests($urpm, $db, $state, \@selected, %options);
+    }
+    @selected;
+}
+
+sub resolve_requested_suggests {
+    my ($urpm, $db, $state, $selected, %options) = @_;
+	my @todo = @$selected;
 	while (@todo) {
 	    my $pkg = shift @todo;
 	    my %suggests = map { $_ => 1 } $pkg->suggests or next;
 
 	    #- do not install a package that has already been suggested
-	    $db->traverse_tag('name', [ $pkg->name ], sub {
+	    $db->traverse_tag_find('name', $pkg->name, sub {
 		my ($p) = @_;
 		delete $suggests{$_} foreach $p->suggests;
 	    });
@@ -908,11 +907,11 @@ sub resolve_requested {
 	    my %new_requested = map { $_ => undef } keys %suggests;
 	    my @new_selected = resolve_requested__no_suggests_($urpm, $db, $state, \%new_requested, %options);
 	    $state->{selected}{$_->id}{suggested} = 1 foreach @new_selected;
-	    push @selected, @new_selected;
+	    push @$selected, @new_selected;
 	    push @todo, @new_selected;
 	}
-    }
-    @selected;
+
+    @$selected;
 }
 
 #- Resolve dependencies of requested packages; keep resolution state to
@@ -937,7 +936,7 @@ sub resolve_requested__no_suggests {
 
     foreach (keys %$requested) {
 	#- keep track of requested packages by propating the flag.
-	foreach (find_candidate_packages_($urpm, $_)) {
+	foreach (find_candidate_packages($urpm, $_)) {
 	    $_->set_flag_requested;
 	}
     }
@@ -968,6 +967,7 @@ sub resolve_requested__no_suggests_ {
     #- then examine provides that will be removed (which need to be satisfied by another
     #- package present or by a new package to upgrade), then requires not satisfied and
     #- finally conflicts that will force a new upgrade or a remove.
+    my $count = 1;
     do {
 	while (my $dep = shift @properties) {
 	    #- we need to avoid selecting packages if the source has been disabled.
@@ -1055,7 +1055,13 @@ sub resolve_requested__no_suggests_ {
 	} elsif (my $dep = shift @choices) {
 	    push @properties, $dep;
 	}
-    } while @diff_provides || @properties || @choices;
+
+	# safety:
+	if ($count++ > 1000) {
+	    die("detecting looping forever while trying to resolve dependancies.\n"
+		. "Aborting... Try again with '-vv --debug' options");
+	}
+    } while (@diff_provides || @properties || @choices);
 
     #- return what has been selected by this call (not all selected hash which may be not empty
     #- previously. avoid returning rejected packages which weren't selectable.
@@ -1071,7 +1077,7 @@ sub resolve_requested__no_suggests_ {
 sub _handle_conflicts_with_selected {
     my ($urpm, $db, $state, $pkg, $dep, $properties, $diff_provides, %options) = @_;
     foreach ($pkg->conflicts) {
-	if (my ($n, $_o, $_v) = property2name_op_version($_)) {
+	if (my $n = property2name($_)) {
 	    foreach my $p ($urpm->packages_providing($n)) {
 		$pkg == $p and next;
 		$p->provides_overlap($_) or next;
@@ -1264,7 +1270,7 @@ sub _handle_diff_provides {
 	#- try if upgrading the package will be satisfying all the requires...
 	#- there is no need to avoid promoting epoch as the package examined is not
 	#- already installed.
-	my @packages = find_candidate_packages_($urpm, $p->name, $state->{rejected});
+	my @packages = find_candidate_packages($urpm, $p->name, $state->{rejected});
 	@packages = 
 	  grep { ($_->name eq $p->name ? $p->compare_pkg($_) < 0 :
 		    $_->obsoletes_overlap($p->name . " == " . $p->epoch . ":" . $p->version . "-" . $p->release))
@@ -1284,7 +1290,7 @@ sub _handle_diff_provides {
 	    #- there exists enough packages that provided the unsatisfied requires.
 	    my @best;
 	    foreach (@unsatisfied) {
-		my @packages = find_candidate_packages_($urpm, $_, $state->{rejected});
+		my @packages = find_candidate_packages($urpm, $_, $state->{rejected});
 		if (@packages = grep { $_->fullname ne $p->fullname } @packages) {
 		    push @best, join('|', map { $_->id } @packages);
 		}
@@ -1324,7 +1330,7 @@ sub _handle_conflict {
     #- whether a newer version will be ok, else ask to remove the old.
     my $need_deps = $p->name . " > " . ($p->epoch ? $p->epoch . ":" : "") .
       $p->version . "-" . $p->release;
-    my @packages = grep { $_->name eq $p->name } find_candidate_packages_($urpm, $need_deps, $state->{rejected});
+    my @packages = grep { $_->name eq $p->name } find_candidate_packages($urpm, $need_deps, $state->{rejected});
     @packages = grep { ! $_->provides_overlap($property) } @packages;
 
     if (!@packages) {
@@ -1612,7 +1618,6 @@ sub compute_flag {
 #-   + those of compute_flag (flag_skip, flag_disable_obsolete)
 sub compute_flags {
     my ($urpm, $val, %options) = @_;
-    if (ref $val eq 'HASH') { $val = [ keys %$val ] } #- compatibility with urpmi <= 4.5-13mdk
     my @regex;
 
     #- unless a regular expression is given, search in provides
@@ -1713,7 +1718,7 @@ sub request_packages_to_upgrade {
 		$pkg = undef;
 	    }
 	}
-	if ($pkg && $options{idlist} && !grep { $pkg->id == $_ } @{$options{idlist}}) {
+	if ($pkg && $options{idlist} && !any { $pkg->id == $_ } @{$options{idlist}}) {
 		$urpm->{debug_URPM}("not auto-selecting " . $pkg->fullname . "because it's not in search medias") if $urpm->{debug_URPM};
 		$pkg = undef;
 	} 
@@ -1803,7 +1808,7 @@ sub sort_graph {
 		# don't care
 	    } elsif (exists $added{$p_id}) {
 		# already done
-	    } elsif (grep { $_ == $p_id } @ids) {
+	    } elsif (any { $_ == $p_id } @ids) {
 		my $begin = 1;
 		my @l = grep { $begin &&= $_ != $p_id } @ids;
 		$loop_ahead = 1;
